import { BadRequestException, ForbiddenException, Injectable, NotFoundException } from '@nestjs/common';
import { CreateOrderDto } from './dto/create-order.dto';
import { OrderRepository } from "./order-repository";
import { ProductsRepository } from "src/modules/product/product/product.repository";
import { InjectRepository } from "@nestjs/typeorm";
import { SalesTaxCMSRepository } from "src/modules/CMS/sales-tax-cms/sales-tax-cms.repository";
import { OrderDetailsRepository } from "../order-details/order-details.repository";
import { ResponseUtils } from "src/utils/response.utils";
import { DirectPost } from 'src/helper/nmi-payment-gatway-helper';
import { PaymentConfigurationRepository } from "src/modules/payment-configuration/payment-configuration.repository";
import { PaymentGatewayRepository } from "src/modules/payment-gateway/payment-gateway.repository";
import { CouponRepository } from "../coupon/coupon.repository";
// import { formattedDateString } from "src/helper/common-helper";
import { MailSenderService } from "src/modules/mail-sender/mail-sender.service";
import { Product } from "src/modules/product/product/entity/product.entity";
import { CreateOrderByAdminDto } from "./dto/create-order-by-admin.dto";
import { UsersRepository } from "src/modules/users/user.repository";
import { convertBase64ToImage, generateRandomPassword } from "src/helper/common-helper";
import CryptoUtils from "src/utils/crypto.utils";
import { User } from "src/modules/users/entities/user.entity";
import { CreateOrderByGuestDto } from "./dto/create-order-by-guest.dto";
import { Constants } from "src/utils/constants";
import { UpsCoreService } from "src/modules/ups/ups-core/ups-core.service";
import { Order } from "./entities/orders.entity";
import { ReturnProductsRepository } from "src/modules/return-products/return-products.repository";
import { RefundItemRepository } from "src/modules/refund-item/refund-item.repository";
import { UpsShippingService } from "src/modules/ups/ups-shipping/ups-shipping.service";
import { DoSpacesService } from "src/modules/SpacesModule/SpacesService/doSpacesService";
import { UpdateOrderDto } from "./dto/update-order.dto";
// import { OrderPlacedDto } from "src/modules/mail-sender/dto/order-placed-mail.dto";

type itemType = {
    productId: string;
    name: string;
    customerId: string;
    orderId: string;
    featuredImage: string;
    productColor: string;
    quantity: number;
    createdBy: string;
    brandName: string;
    itemSubTotal: number;
    itemTotal: number;
    itemPrice: number;
}

@Injectable()
export class OrderService {
    constructor(
        @InjectRepository(OrderRepository) private readonly repository: OrderRepository,
        @InjectRepository(CouponRepository) private readonly couponsRepository: CouponRepository,
        @InjectRepository(ProductsRepository) private readonly productRepository: ProductsRepository,
        @InjectRepository(SalesTaxCMSRepository) private readonly salesTaxCMSRepository: SalesTaxCMSRepository,
        @InjectRepository(OrderDetailsRepository) private readonly orderDetailsRepository: OrderDetailsRepository,
        @InjectRepository(PaymentConfigurationRepository) private readonly paymentConfigurationRepository: PaymentConfigurationRepository,
        @InjectRepository(PaymentGatewayRepository) private readonly paymentGatewayRepository: PaymentGatewayRepository,
        @InjectRepository(UsersRepository) private readonly userRepository: UsersRepository,
        @InjectRepository(ReturnProductsRepository) private readonly returnProductsRepo: ReturnProductsRepository,
        @InjectRepository(RefundItemRepository) private readonly refundItemRepository: RefundItemRepository,
        // @InjectRepository(FooterCmsRepository) private readonly footerCmsRepository: FooterCmsRepository,
        private readonly servMail: MailSenderService,
        private readonly upsCore: UpsCoreService,
        private readonly upsShipping: UpsShippingService,
        private readonly doSpacesService: DoSpacesService
    ) { }

    private async getProducts(products: any[]) {
        try {
            const productIds = products.map((x: { id: string }) => x.id);
            const productList = await this.productRepository.findByIds(productIds);

            const productMap = new Map<string, any>();
            for (const x of products) {
                const item: any = x;
                productMap.set(item?.id, item);
            }

            return {
                products: productList,
                productMap
            }
        } catch (error) {
            throw error;
        }
    }

    private generateProductDetails(products: Product[], productMap: Map<string, any>, user: any, orderId: string, customerId: string) {
        try {
            let subTotal = 0;
            let totalQuantity = 0;

            const items = products.map((item) => {
                const itemInfo = productMap.get(item?.id);

                const price = +item.discountPrice > 0 ? +item.discountPrice : +item.price;

                if (itemInfo.quantity > 0) {
                    totalQuantity += (+itemInfo.quantity);
                    subTotal += (+itemInfo.quantity) * +price;
                }

                //details of items object populate
                let details = {
                    productId: itemInfo.id,
                    name: item.name,
                    customerId: customerId,
                    orderId: orderId,
                    featuredImage: item.featuredImage,
                    productColor: itemInfo.productColor ? itemInfo.productColor.toString() : null,
                    quantity: +itemInfo.quantity,
                    createdBy: user.id,
                    brandName: item.brandName,
                    itemSubTotal: +itemInfo.itemSubTotal,
                    itemTotal: +itemInfo.quantity * +price,
                    itemPrice: +price
                };

                return details;
            });

            return {
                subTotal,
                totalQuantity,
                items
            }
        } catch (error) {
            throw error;
        }
    }

    private async getcouponDiscount(coupon: string, subTotal: number) {
        try {
            let couponDiscount = 0;
            if (coupon) {
                couponDiscount = await this.couponsRepository.getCouponDiscountByCouponNumber(coupon, subTotal);
                if (!couponDiscount) {
                    throw new BadRequestException('Coupon is not valid!');
                }
            }
            return couponDiscount;
        } catch (error) {
            throw error;
        }
    }

    private async placeOrderByUser(
        dto: CreateOrderDto,
        subTotal: number,
        orderId: string,
        user: any,
        couponDiscount: number,
        totalQuantity: number,
        items: itemType[]
    ) {
        try {
            const tax = await this.salesTaxCMSRepository.findOne();
            const salesTaxAmount = subTotal * tax.salesTax / 100;

            const placeOrder = {
                orderId: orderId,
                customerId: user.id,
                customerName: user.firstName + user.lastName,
                couponNumber: dto.coupon ? dto.coupon : null,
                couponDiscount,
                totalQuantity,
                subTotal: +(subTotal.toFixed(2)),
                total: +((+subTotal + (+dto.shippingCost) + salesTaxAmount - couponDiscount).toFixed(2)),
                paymentMethodName: dto.paymentMethodName ? dto.paymentMethodName : null,
                shippingAddress: JSON.stringify(dto.shippingAddress),
                billingAddress: JSON.stringify(dto.billingAddress),
                shippingCost: dto.shippingCost,
                shippingType: dto.shippingType,
                serviceType: dto.serviceType,
                totalWeight: dto.weightValue,
                createdBy: user.id,
                discountType: dto.discountType,
                discountTotal: dto.discountTotal,
                salesTaxTitle: dto.salesTaxTitle,
                salesTaxRate: dto.salesTaxRate,
                salesTaxAmount,
                cardNumber: `************${dto.ccNum.slice(-4)}`,
            };

            const orderPlaced = await this.repository.createEntity(placeOrder);
            await this.orderDetailsRepository.createBulkEntity(items);

            return orderPlaced;
        } catch (error) {
            throw error
        }
    }

    private async placeOrderByAdmin(
        dto: CreateOrderByAdminDto,
        subTotal: number,
        orderId: string,
        user: any,
        couponDiscount: number,
        totalQuantity: number,
        items: itemType[],
        orderFor: User
    ) {
        try {
            const tax = await this.salesTaxCMSRepository.findOne();
            const salesTaxAmount = subTotal * tax.salesTax / 100;

            const placeOrder = {
                orderId: orderId,
                customerId: orderFor.id,
                customerName: orderFor.firstName + ' ' + (orderFor.lastName ? orderFor.lastName : ''),
                couponNumber: dto.coupon ? dto.coupon : null,
                couponDiscount,
                totalQuantity,
                subTotal: +(subTotal.toFixed(2)),
                total: +((+subTotal + (+dto.shippingCost) + salesTaxAmount - couponDiscount).toFixed(2)),
                paymentMethodName: dto.paymentMethodName ? dto.paymentMethodName : null,
                shippingAddress: JSON.stringify(dto.shippingAddress),
                shippingCost: dto.shippingCost,
                shippingType: dto.shippingType,
                serviceType: dto.serviceType,
                totalWeight: dto.weightValue,
                createdBy: user.id,
                discountType: dto.discountType,
                discountTotal: dto.discountTotal,
                salesTaxTitle: dto.salesTaxTitle,
                salesTaxRate: dto.salesTaxRate,
                salesTaxAmount,
                cardNumber: `************${dto.ccNum.slice(-4)}`,
                paymentStatus: dto.paymentStatus,
            };
            const orderPlaced = await this.repository.createEntity(placeOrder);
            await this.orderDetailsRepository.createBulkEntity(items);

            return orderPlaced;
        } catch (error) {
            throw error;
        }
    }

    private async placeOrderByGuest(
        dto: CreateOrderByGuestDto,
        subTotal: number,
        orderId: string,
        user: { guestId: string; guestName: string },
        couponDiscount: number,
        totalQuantity: number,
        items: itemType[]
    ) {
        try {
            const tax = await this.salesTaxCMSRepository.findOne();
            const salesTaxAmount = subTotal * tax.salesTax / 100;

            const placeOrder = {
                orderId: orderId,
                customerId: user.guestId,
                customerName: user.guestName,
                couponNumber: dto.coupon ? dto.coupon : null,
                couponDiscount,
                totalQuantity,
                subTotal: +(subTotal.toFixed(2)),
                total: +((+subTotal + (+dto.shippingCost) + salesTaxAmount - couponDiscount).toFixed(2)),
                paymentMethodName: dto.paymentMethodName ? dto.paymentMethodName : null,
                shippingAddress: JSON.stringify(dto.shippingAddress),
                billingAddress: JSON.stringify(dto.billingAddress),
                shippingCost: dto.shippingCost,
                shippingType: dto.shippingType,
                serviceType: dto.serviceType,
                totalWeight: dto.weightValue,
                createdBy: user.guestId,
                // discountType: dto.discountType,
                // discountTotal: dto.discountTotal,
                salesTaxTitle: dto.salesTaxTitle,
                salesTaxRate: dto.salesTaxRate,
                salesTaxAmount,
                cardNumber: `************${dto.ccNum.slice(-4)}`,
            };

            const orderPlaced = await this.repository.createEntity(placeOrder);
            await this.orderDetailsRepository.createBulkEntity(items);

            return orderPlaced;
        } catch (error) {
            throw error;
        }
    }

    private async createPaymentByUser(dto: CreateOrderDto, user: any, orderId: string, orderPlaced: any) {
        try {
            if (dto.paymentMethodName === 'Credit/Debit Card') {
                const paymentConf = await this.paymentConfigurationRepository.find();
                if (!paymentConf) {
                    throw new NotFoundException('Payment security key not found');
                }
                const dp = new DirectPost(paymentConf[0].securityKey);
                const billingData: any = dto.billingAddress ? dto.billingAddress : {};
                const shippingData: any = dto.shippingAddress ? dto.shippingAddress : {};
                const billingName = billingData.name.split(' ');
                const shippingName = shippingData.name.split(' ');
                const billingInfo = {
                    first_name: billingName[0],
                    last_name: billingName.length > 1 ? billingName[1] : '',
                    address1: billingData.address,
                    city: billingData.city,
                    state: billingData.stateCode,
                    zip: billingData.postCode,
                };
                const shippingInfo = {
                    shipping_first_name: shippingName[0],
                    shipping_last_name: shippingName.length > 1 ? shippingName[1] : '',
                    shipping_address1: shippingData.addressOne,
                    shipping_city: shippingData.city,
                    shipping_state: shippingData.stateCode,
                    shipping_zip: shippingData.postCode,
                };

                dp.setBilling(billingInfo);
                dp.setShipping(shippingInfo);
                const data = await dp.doSale(
                    dto.amount,
                    dto.ccNum,
                    dto.ccExp,
                    dto.cvv,
                );

                if (
                    data['statusCode'] === 200 &&
                    data['response'] === '1' &&
                    data['response_code'] === '100' &&
                    data['transactionid']
                ) {
                    const payload = {
                        customerId: user.id,
                        customerName: user.firstName + ' ' + user.lastName,
                        invoiceID: orderId,
                        paymentAmount: dto.amount,
                        paymentDate: new Date(),
                        paymentTransID: data['transactionid'],
                        paymentPayload: JSON.stringify(data),
                        transStatus: 'Success',
                    };
                    const res = await this.paymentGatewayRepository.createEntity(payload);

                    // update payment status
                    const updateOrder = await this.repository.update(
                        orderPlaced['id'],
                        { paymentStatus: 'Paid' },
                    );

                    // const footerCMS = await this.footerCmsRepository.findAll();
                    // let mailDto = {
                    //     to: [user.email],
                    //     from: process.env.EMAIL_SENDER_MAIL,
                    //     subject: 'Order Placed Notification',
                    //     customerName: user.firstName + ' ' + user.lastName,
                    //     orderData: orderPlaced,
                    //     totalAmount: orderPlaced.total,
                    //     orderDate: formattedDateString(orderPlaced.createdAt),
                    //     shippingAddress: orderPlaced.shippingAddress,
                    //     contactInfo: footerCMS ? footerCMS[0] : {},
                    //     items: items as any,
                    // } as OrderPlacedDto;

                    // const mail = await this.servMail.sendPlaceOrderNotification(mailDto);

                    return ResponseUtils.successResponseHandler(
                        200,
                        'Order placed and payment processed successfully.',
                        'orderId',
                        orderId,
                    );
                } else {
                    // delete order
                    const orderDetails = await this.orderDetailsRepository.find({
                        orderId: orderPlaced['orderId'],
                    });
                    if (orderDetails.length > 0) {
                        orderDetails.forEach(async (element: any) => {
                            await this.orderDetailsRepository.remove(element);
                        });
                    }
                    await this.repository.remove(orderPlaced);
                    return ResponseUtils.errorResponseHandler(
                        400,
                        'Payment failed! Please try again',
                        'errors',
                        data,
                    );
                }
            } else {
                // const footerCMS = await this.footerCmsRepository.findAll();
                // const mailDto = {
                //     to: [user.email],
                //     from: process.env.EMAIL_SENDER_MAIL,
                //     subject: 'Order Placed Notification',
                //     customerName: user.firstName + ' ' + user.lastName,
                //     orderData: orderPlaced,
                //     totalAmount: (+orderPlaced.total) + (+orderPlaced.shippingCost) + (+orderPlaced.salesTaxAmount) - (+orderPlaced.totalDiscount),
                //     orderDate: formattedDateString(orderPlaced.createdAt),
                //     shippingAddress: orderPlaced.shippingAddress,
                //     contactInfo: footerCMS ? footerCMS[0] : {},
                //     items: items as any
                // } as OrderPlacedDto;

                // const mail = await this.servMail.sendPlaceOrderNotification(mailDto);

                return ResponseUtils.successResponseHandler(
                    200,
                    'Order placed successfully.',
                    'orderId',
                    orderId,
                );
            }
        } catch (error) {
            throw error;
        }
    }

    private async createPaymentByAdmin(dto: CreateOrderByAdminDto, orderFor: User, orderId: string, orderPlaced: any) {
        try {
            if (dto.paymentMethodName === 'Credit/Debit Card') {
                const paymentConf = await this.paymentConfigurationRepository.find();
                if (!paymentConf) {
                    throw new NotFoundException('Payment security key not found');
                }
                const dp = new DirectPost(paymentConf[0].securityKey);
                const billingData: any = dto.shippingAddress ? dto.shippingAddress : {};
                const billingName = billingData.name;
                const billingInfo = {
                    first_name: billingName,
                    address1: billingData.addressOne,
                    city: billingData.city,
                    state: billingData.stateCode,
                    zip: billingData.postCode,
                };
                dp.setBilling(billingInfo);
                const data = await dp.doSale(
                    dto.amount,
                    dto.ccNum,
                    dto.ccExp,
                    dto.cvv,
                );

                if (
                    data['statusCode'] === 200 &&
                    data['response'] === '1' &&
                    data['response_code'] === '100' &&
                    data['transactionid']
                ) {
                    const payload = {
                        customerId: orderFor.id,
                        customerName: orderFor.firstName + ' ' + orderFor.lastName,
                        invoiceID: orderId,
                        paymentAmount: dto.amount,
                        paymentDate: new Date(),
                        paymentTransID: data['transactionid'],
                        paymentPayload: JSON.stringify(data),
                        transStatus: 'SUCCESS',
                    };
                    const res = await this.paymentGatewayRepository.createEntity(
                        payload,
                    );

                    // let mailAdminDto = {
                    //     to: [orderFor.email],
                    //     from: process.env.EMAIL_SENDER_MAIL,
                    //     subject: 'Order Placed Notification.',
                    //     customerName: user.firstName + ' ' + user.lastName,
                    //     orderData: orderPlaced,
                    //     totalAmount: parseFloat(orderPlaced.total),
                    //     listingOwner: user.fullName,
                    //     listingStatus: 'Pending',
                    //     listingID: orderId,
                    //     shippingAddress: orderPlaced.shippingAddress,
                    //     items: items,
                    // };
                    // const adminMail = await this.servMail.sendNewListingNotification(
                    //     mailAdminDto as CreateListingStatusDto,
                    // );

                    return ResponseUtils.successResponseHandler(
                        200,
                        'Order Placed and Payment successful',
                        'orderId',
                        orderId,
                    );
                } else {
                    await this.repository.hardDeleteEntity(
                        orderPlaced.id,
                        orderPlaced.orderId,
                    );
                    return ResponseUtils.errorResponseHandler(
                        400,
                        'Payment failed! Please try again',
                        'errors',
                        data,
                    );
                }
            } else {
                // let mailAdminDto = {
                //     to: [orderFor.email],
                //     from: process.env.EMAIL_SENDER_MAIL,
                //     subject: 'Order Placed Notification.',
                //     customerName: user.firstName + ' ' + user.lastName,
                //     orderData: orderPlaced,
                //     totalAmount: parseFloat(orderPlaced.total),
                //     listingOwner: user.fullName,
                //     listingStatus: 'Pending',
                //     listingID: orderId,
                //     shippingAddress: orderPlaced.shippingAddress,
                //     items: items,
                // };
                // const adminMail = await this.servMail.sendNewListingNotification(
                //     mailAdminDto as CreateListingStatusDto,
                // );
                return ResponseUtils.successResponseHandler(
                    200,
                    'Order created successfully',
                    'orderId',
                    orderId,
                );
            }
        } catch (error) {
            throw error;
        }
    }

    private async createPaymentByGuest(dto: CreateOrderByGuestDto, user: { guestId: string; guestName: string }, orderId: string, orderPlaced: any) {
        try {
            if (dto.paymentMethodName === 'Credit/Debit Card') {
                const paymentConf = await this.paymentConfigurationRepository.find();
                if (!paymentConf) {
                    throw new NotFoundException('Payment security key not found');
                }
                const dp = new DirectPost(paymentConf[0].securityKey);
                const billingData: any = dto.billingAddress ? dto.billingAddress : {};
                const shippingData: any = dto.shippingAddress ? dto.shippingAddress : {};
                const billingName = billingData.name.split(' ');
                const shippingName = shippingData.name.split(' ');
                const billingInfo = {
                    first_name: billingName[0],
                    last_name: billingName.length > 1 ? billingName[1] : '',
                    address1: billingData.address,
                    city: billingData.city,
                    state: billingData.stateCode,
                    zip: billingData.postCode,
                };
                const shippingInfo = {
                    shipping_first_name: shippingName[0],
                    shipping_last_name: shippingName.length > 1 ? shippingName[1] : '',
                    shipping_address1: shippingData.addressOne,
                    shipping_city: shippingData.city,
                    shipping_state: shippingData.stateCode,
                    shipping_zip: shippingData.postCode,
                };
                dp.setBilling(billingInfo);
                dp.setShipping(shippingInfo);
                const data = await dp.doSale(
                    dto.amount,
                    dto.ccNum,
                    dto.ccExp,
                    dto.cvv,
                );
                if (
                    data['statusCode'] === 200 &&
                    data['response'] === '1' &&
                    data['response_code'] === '100' &&
                    data['transactionid']
                ) {
                    const payload = {
                        customerId: user.guestId,
                        customerName: user.guestName,
                        invoiceID: orderId,
                        paymentAmount: dto.amount,
                        paymentDate: new Date(),
                        paymentTransID: data['transactionid'],
                        paymentPayload: JSON.stringify(data),
                        transStatus: 'Success',
                    };
                    const res = await this.paymentGatewayRepository.createEntity(
                        payload,
                    );
                    // Update payment status
                    const updateOrder = await this.repository.update(
                        orderPlaced['id'],
                        { paymentStatus: 'Paid' },
                    );
                    // Mail to user and admin
                    // const billingData: any = dto.billingAddress ? dto.billingAddress : {};
                    // const shippingData: any = dto.shippingAddress ? dto.shippingAddress : {};
                    // let mailAdminDto = {
                    //     to: [billingData.email, shippingData.email],
                    //     from: process.env.EMAIL_SENDER_MAIL,
                    //     subject: 'Order Placed Notification.',
                    //     listingOwner: billingData.name,
                    //     listingStatus: 'Pending',
                    //     listingID: orderId,
                    // };
                    // const adminMail = await this.servMail.sendNewListingNotification(
                    //     mailAdminDto as CreateListingStatusDto,
                    // );
                    return ResponseUtils.successResponseHandler(
                        200,
                        'Order placed and payment successfully completed.',
                        'orderId',
                        orderId,
                    );
                } else {
                    // Delete order
                    const orderDetails = await this.orderDetailsRepository.find({
                        orderId: orderPlaced['orderId'],
                    });
                    if (orderDetails.length > 0) {
                        orderDetails.forEach(async (element: any) => {
                            await this.orderDetailsRepository.remove(element);
                        });
                    }
                    await this.repository.remove(orderPlaced);
                    return ResponseUtils.errorResponseHandler(
                        400,
                        'Payment failed! Please try again.',
                        'errors',
                        data,
                    );
                }
            } else {
                // const billingData: any = dto.billingAddress ? dto.billingAddress : {};
                // const shippingData: any = dto.shippingAddress ? dto.shippingAddress : {};
                // let mailAdminDto = {
                //     to: [billingData.email, shippingData.email],
                //     from: process.env.EMAIL_SENDER_MAIL,
                //     subject: 'Order Placed Notification.',
                //     listingOwner: billingData.name,
                //     listingStatus: 'Pending',
                //     listingID: orderId,
                // };
                // const adminMail = await this.servMail.sendNewListingNotification(
                //     mailAdminDto as CreateListingStatusDto,
                // );
                return ResponseUtils.successResponseHandler(
                    200,
                    'Order created successfully.',
                    'orderId',
                    orderId,
                );
            }
        } catch (error) {
            throw error;
        }
    }

    async create(dto: CreateOrderDto, user: any): Promise<any | Error> {
        try {
            const { products, productMap } = await this.getProducts(dto.products);
            const orderId = await this.getOrderId();

            const { subTotal, totalQuantity, items } = this.generateProductDetails(products, productMap, user, orderId, user.id);
            const couponDiscount = await this.getcouponDiscount(dto.coupon, subTotal);
            const orderPlaced = await this.placeOrderByUser(dto, subTotal, orderId, user, couponDiscount, totalQuantity, items);

            if (orderPlaced) {
                return await this.createPaymentByUser(dto, user, orderId, orderPlaced);
            } else {
                return ResponseUtils.errorResponseHandler(400, 'Failed to create order data!');
            }
        } catch (error) {
            throw error;
        }
    }

    async createOrderByAdmin(dto: CreateOrderByAdminDto, user: any): Promise<any | Error> {
        try {
            const { products, productMap } = await this.getProducts(dto.products);
            const orderId = await this.getOrderId();

            let orderFor = await this.userRepository.findOne({
                where: { email: dto.email },
            });

            if (!orderFor) {
                const generateRandomPass = generateRandomPassword(8);
                const userDto = {
                    email: dto.email,
                    firstName: dto.customerName,
                    lastName: '',
                    fullAddress: null,
                    dateOfBirth: null,
                    gender: null,
                    city: null,
                    postalCode: null,
                    country: null,
                    mobile: null,
                    phone: null,
                    image: null,
                    company: null,
                    zipCode: null,
                    userType: 'customer',
                    role: 3,
                    password: await CryptoUtils.encrypt(generateRandomPass),
                    discountType: dto.discountType,
                    discountAmount: dto.discountAmount,
                };

                const createUser = await this.userRepository.createEntity(userDto);
                if (createUser) orderFor = createUser as User;

                if (orderFor) {
                    // const footerCMS = await this.footerCmsRepository.findAll();
                    // const headerCMS = await this.headerCmsRepository.findAll();
                    // const mailDto = {
                    //     to: [dto.email],
                    //     from: process.env.EMAIL_SENDER_MAIL,
                    //     subject: 'Hyman fashion User Account Creation Notification',
                    //     username: dto.customerName,
                    //     password: generateRandomPass,
                    //     contactInfo: footerCMS ? footerCMS[0] : {},
                    //     headerCMS: headerCMS ? headerCMS[0] : {},
                    // };

                    // const mail = await this.servMail.accocuntCreateByAdminCustomerNotify(
                    //     mailDto as CreateMailSenderDto,
                    // );
                }
            } else if (orderFor && orderFor.isDeleted === true) {
                await this.userRepository.update(orderFor.id, {
                    isDeleted: false,
                });
            }

            const { subTotal, totalQuantity, items } = this.generateProductDetails(products, productMap, user, orderId, orderFor.id);
            const couponDiscount = await this.getcouponDiscount(dto.coupon, subTotal);
            const orderPlaced = await this.placeOrderByAdmin(dto, subTotal, orderId, user, couponDiscount, totalQuantity, items, orderFor);

            if (orderPlaced) {
                return await this.createPaymentByAdmin(dto, orderFor, orderId, orderPlaced);
            } else {
                return ResponseUtils.errorResponseHandler(400, 'Failed to create order data!');
            }
        } catch (error) {
            throw error;
        }
    }

    async createOrderByGuest(dto: CreateOrderByGuestDto): Promise<any | Error> {
        try {
            const guestId = '9999999999999';
            const guestName = 'Guest';

            const { products, productMap } = await this.getProducts(dto.products);
            const orderId = await this.getOrderId();

            const { subTotal, totalQuantity, items } = this.generateProductDetails(products, productMap, { id: guestId }, orderId, guestId);
            const couponDiscount = await this.getcouponDiscount(dto.coupon, subTotal);
            const orderPlaced = await this.placeOrderByGuest(dto, subTotal, orderId, { guestId, guestName }, couponDiscount, totalQuantity, items);

            if (orderPlaced) {
                return await this.createPaymentByGuest(dto, { guestId, guestName }, orderId, orderPlaced);
            } else {
                return ResponseUtils.errorResponseHandler(400, 'Failed to create order data!');
            }
        } catch (error) {
            throw error;
        }
    }

    async getOrderId(): Promise<string> {
        try {
            const countQuery = `select count(*) count from orders`;
            const orderCount = await this.repository.query(countQuery);
            const prefix = "GC-";
            const prefixCount = prefix.length + 1;
            if (orderCount?.[0]?.count > 0) {
                const orderIdGenerateQuery =
                    `SELECT SUBSTRING(r.orderId, ${prefixCount}) AS orderId FROM orders AS r
                ORDER BY SUBSTRING(r.orderId, ${prefixCount}) DESC
                LIMIT 1;`;

                const data = await this.repository.query(orderIdGenerateQuery);
                if (data?.[0]?.orderId) {
                    const orderId = data?.[0]?.orderId;
                    const id = (+orderId + 1).toString().padStart(6, '0');
                    return `${prefix}${id}`;
                }
            }

            return 'GC-000001';
        } catch (error) {
            return 'GC-000001';
        }
    }

    async validateShipping(dto: any): Promise<any | Error> {
        try {
            let data: any;
            const order = await this.repository.findOne(dto.id);
            if (!order) {
                throw new NotFoundException(Constants.NOT_FOUND);
            }
            const orderId = order.id;
            const shipAddress = JSON.parse(order.shippingAddress);

            if (order.shippingType === 'ups') {
                const payload = {
                    shippingAddress: {
                        addressOne: shipAddress.addressOne,
                        addressTwo: shipAddress.addressTwo,
                        city: shipAddress.city,
                        stateCode: shipAddress.stateCode,
                        postalCode: shipAddress.postCode,
                        countryCode: shipAddress.countryCode,
                    },
                    serviceType: order.serviceType,
                    weightValue: order.totalWeight,
                };
                data = await this.upsCore.pickupCreate(payload);
                if (data['PickupCreationResponse'].Response.ResponseStatus.Description === 'Success') {
                    await this.repository.updateOrderStatus(orderId, 'SHIPPED');
                }
            }
            return ResponseUtils.successResponseHandler(200, '', 'data', data);
        } catch (err) {
            throw err;
        }
    }

    async updateOrderTracking(dto: any): Promise<any | Error> {
        try {
            const order = await this.repository.findOne({
                where: {
                    orderId: dto.id,
                },
            });
            if (!order) {
                throw new NotFoundException(Constants.NOT_FOUND);
            }
            const orderId = order.id;

            await this.repository.update(orderId, {
                trackingNumber: dto.trackingNumber,
            });

            const datas = await this.repository.findOne({
                where: {
                    orderId: dto.id,
                },
            });
            if (!datas) {
                throw new NotFoundException(Constants.NOT_FOUND);
            }

            return ResponseUtils.successResponseHandler(
                200,
                'Order info updated',
                'data',
                datas,
            );
        } catch (err) {
            throw err;
        }
    }

    async findAll(
        take: number,
        page: number,
        paymentStatus?: string,
        deliveryStatus?: string,
        startDate?: Date,
        endDate?: Date,
        customerName?: string,
        orderId?: string,
    ): Promise<Order[] | Error> {
        const query: any = {};
        if (paymentStatus) query.paymentStatus = paymentStatus;
        if (deliveryStatus) query.deliveryStatus = deliveryStatus;
        if (startDate) query.startDate = startDate;
        if (endDate) query.endDate = endDate;
        if (customerName) query.customerName = customerName;
        if (orderId) query.orderId = orderId;

        const data = await this.repository.findAllEntities(take, page, query);
        if (!data) {
            throw new NotFoundException(Constants.NOT_FOUND);
        }
        return ResponseUtils.successResponseHandler(
            200,
            'Orders data successfully fetched.',
            'data',
            data,
        );
    }

    async findOrderByFilter(
        authUser: any,
        limit: number,
        page: number,
        tabStatus: string,
    ): Promise<any | Error> {
        if (!authUser) {
            throw new ForbiddenException(Constants.UNAUTH_REQ);
        }

        if (tabStatus === 'totalOrders') {
            return await this.repository.totalUserOrderList(limit, page, authUser.user.id);
        }

        if (tabStatus === 'pending') {
            return await this.repository.totalUserPendingOrderList(limit, page, authUser.user.id, 'PENDING');
        }

        if (tabStatus === 'shipped') {
            return await this.repository.totalUserShippedOrderList(limit, page, authUser.user.id, 'SHIPPED');
        }

        if (tabStatus === 'delivered') {
            return await this.repository.totalUserDeliveredOrderList(limit, page, authUser.user.id, 'DELIVERED');
        }

        if (tabStatus === 'cancelled') {
            return await this.repository.totalUserCancelledOrderList(limit, page, authUser.user.id, 'CANCELLED');
        }

        if (tabStatus === 'processing') {
            return await this.repository.totalUserProcessingOrderList(limit, page, authUser.user.id, 'PROCESSING');
        }
    }

    async orderProfitService(
        take: number,
        page: number,
        startDate?: Date,
        endDate?: Date,
        orderId?: string,
    ): Promise<Order[] | Error> {
        const query: any = {};
        if (startDate) query.startDate = startDate;
        if (endDate) query.endDate = endDate;
        if (orderId) query.orderId = orderId;

        const data = await this.orderDetailsRepository.findAllEntities(take, page, query);
        if (!data) {
            throw new NotFoundException(Constants.NOT_FOUND);
        }
        return ResponseUtils.successResponseHandler(
            200,
            'Orders data successfully fetched.',
            'data',
            data,
        );
    }

    async checkOrderId(orderId: string): Promise<any | Error> {
        const data = await this.repository.findOne({
            where: {
                orderId: orderId,
                isDeleted: false,
            },
        });

        if (data) {
            return true;
        } else {
            return false;
        }
    }

    async findOne(id: string): Promise<Order | Error> {
        const data = await this.repository.findOneEntity(id);
        if (Object.keys(data).length === 0) {
            throw new NotFoundException(Constants.NOT_FOUND);
        }

        const user = await this.userRepository.findOne((data?.[0] as Order)?.customerId);
        data[0]['user'] = user;

        const paymentGateway = await this.paymentGatewayRepository.findOne({
            where: {
                invoiceID: data[0]['orderId'],
                isRefunded: true,
                isDeleted: false,
            },
        });
        if (paymentGateway) {
            if (
                parseFloat(paymentGateway.paymentAmount) == paymentGateway.refundAmount
            ) {
                data[0]['isGlobalRefunded'] = true;
                data[0]['refundAmount'] = paymentGateway.refundAmount;
            } else {
                data[0]['isGlobalRefunded'] = false;
                data[0]['refundAmount'] = paymentGateway.refundAmount;
            }
        } else {
            data[0]['isGlobalRefunded'] = false;
            data[0]['refundAmount'] = 0;
        }

        const orderDetails = data[0]['orders'];
        for (let i = 0; i < data[0]['orders'].length; i++) {
            // return iterms
            const returnItem = await this.returnProductsRepo.find({
                where: {
                    orderId: orderDetails[i]['orderId'],
                    productId: orderDetails[i]['productId'],
                    isDeleted: false,
                },
            });

            let returnQuantity = 0;
            let returnStatus = '';
            for (let j = 0; j < Object.keys(returnItem).length; j++) {
                if (
                    returnItem[j].status === 'Approved' ||
                    returnItem[j].status == 'Unapproved'
                ) {
                    returnQuantity += returnItem[j].quantity;
                    returnStatus = returnItem[j].status;
                }
            }
            const refundItem = await this.refundItemRepository.checkOrderIsRefunded(
                orderDetails[i]['orderId'],
                orderDetails[i]['productId'],
            );
            if (refundItem) {
                data[0]['orders'][i]['isRefunded'] = true;
                data[0]['orders'][i]['returnQuantity'] = returnQuantity;
                data[0]['orders'][i]['returnStatus'] = returnStatus;
            } else {
                data[0]['orders'][i]['isRefunded'] = false;
                data[0]['orders'][i]['returnQuantity'] = returnQuantity;
                data[0]['orders'][i]['returnStatus'] = returnStatus;
            }
        }
        return ResponseUtils.successResponseHandler(
            200,
            'Order data successfully fetched',
            'Orders',
            data,
        );
    }

    async createShippingLabel(id: string, dto: any): Promise<any | Error> {
        try {
            const order = await this.repository.findOne(id);
            if (!order) {
                throw new NotFoundException(Constants.NOT_FOUND);
            }

            const orderShippingType = 'ups';

            // update order table
            await this.repository.update(id, {
                labelStockType: dto.labelStockType,
                labelImageType: dto.labelImageType,
            });
            const data = await this.repository.findOne(id);
            if (!data) {
                throw new NotFoundException(Constants.NOT_FOUND);
            }

            const shipAddress = JSON.parse(order.shippingAddress);
            if (orderShippingType === 'ups') {
                const dto = {
                    shippingAddress: {
                        name: shipAddress.name,
                        mobile: shipAddress.mobile,
                        addressOne: shipAddress.addressOne,
                        addressTwo: shipAddress.addressTwo,
                        city: shipAddress.city,
                        stateCode: shipAddress.stateCode,
                        postCode: shipAddress.postCode,
                        countryCode: shipAddress.countryCode,
                    },
                    serviceType: order.serviceType,
                    weightValue: order.totalWeight,
                };
                const upsShipping = await this.upsShipping.create(dto);

                if (upsShipping?.ShipmentResponse?.ShipmentResults?.PackageResults) {
                    const upsData =
                        upsShipping?.ShipmentResponse?.ShipmentResults?.PackageResults;
                    if (upsData?.ShippingLabel?.ImageFormat?.Code === 'PNG') {
                        const base64ToImage = convertBase64ToImage(upsData?.ShippingLabel?.GraphicImage);
                        const image: any = await this.doSpacesService.uploadFileBuffer(
                            base64ToImage,
                            `${upsData?.ShippingLabel?.ImageFormat?.Code.toLowerCase()}`,
                            'uploads/FedExLabelImage/',
                        );

                        await this.repository.update(id, {
                            trackingNumber: upsData?.TrackingNumber,
                            labelImageEncode: image,
                        });
                    }
                } else {
                    return ResponseUtils.errorResponseHandler(
                        upsShipping?.Response?.ResponseStatus?.Code,
                        'Shipping Label Create Failed!',
                        'data',
                        upsShipping,
                    );
                }
            }
            return ResponseUtils.successResponseHandler(
                200,
                'Order data updated!',
                'data',
                data,
            );
        } catch (err) {
            throw err;
        }
    }

    async update(id: string, dto: UpdateOrderDto): Promise<Order | Error> {
        try {
            const order = await this.repository.findOne(id);
            if (!order) {
                throw new NotFoundException(Constants.NOT_FOUND);
            }

            const data = await this.repository.updateOrder(id, dto);
            if (!data) {
                throw new NotFoundException(Constants.NOT_FOUND);
            }

            // Create payment log
            if (dto.paymentStatus === 'Paid') {
                const payload = {
                    customerId: order.id,
                    customerName: order.customerName,
                    invoiceID: order.orderId,
                    paymentAmount:
                        Number(order.subTotal) +
                        Number(order.salesTaxAmount) +
                        Number(order.shippingCost),
                    paymentDate: new Date(),
                    transStatus: 'Success',
                };
                const res = await this.paymentGatewayRepository.createEntity(payload);
                // const updateOrder = await this.orderRepository.update(orderPlaced['id'], { paymentStatus: 'PAID' });
            }

            if (order['status'] != dto.status) {
                const customerEmail = await this.userRepository.findOne(
                    data['customerId'],
                );

                // send mail
                const proGiftItems: any = [];
                // fetch order details
                const orderDetails = await this.orderDetailsRepository.find({
                    where: {
                        orderId: order.orderId,
                        isDeleted: false,
                    },
                });

                for (let i = 0; i < Object.keys(orderDetails).length; i++) {
                    const data = {
                        name: orderDetails[i].name,
                        quantity: orderDetails[i].quantity,
                        itemPrice: orderDetails[i].itemPrice,
                        featuredImage: orderDetails[i].featuredImage,
                    };
                    proGiftItems.push(data);
                }

                // const footerCMS = await this.footerCmsRepository.findAll();
                // const userEmail = customerEmail['email'];
                // let mailDto = {
                //     to: [userEmail],
                //     from: process.env.EMAIL_SENDER_MAIL,
                //     subject: 'Order Status Changed Notification',
                //     customerName: data['customerName'],
                //     orderData: data,
                //     totalAmount: parseFloat(data["total"]),
                //     orderDate: formattedDateString(data['createdAt']),
                //     shippingAddress: JSON.parse(data['shippingAddress']),
                //     contactInfo: footerCMS ? footerCMS[0] : {},
                //     items: proGiftItems
                // };

                // const mail = await this.servMail.sendOrderStatusChangeNotification(
                //     mailDto as unknown as OrderStatusChangeDto,
                // );
            }
            return ResponseUtils.successResponseHandler(
                200,
                'Order data updated!',
                'data',
                data,
            );
        } catch (err) {
            throw err;
        }
    }

    async remove(id: string): Promise<boolean | Error> {
        const deleteOrder = await this.repository.deleteEntity(id);
        if (!deleteOrder) {
            throw new NotFoundException(Constants.NOT_FOUND);
        }

        const deleteOrderDetails = await this.orderDetailsRepository.deleteEntity(
            id,
        );

        if (deleteOrder && deleteOrderDetails) {
            return ResponseUtils.buildDeletedData(true);
        } else {
            return ResponseUtils.buildDeletedData(false);
        }
    }
}
