import { HttpStatus, Injectable, NotFoundException } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { DirectPost } from 'src/helper/nmi-payment-gatway-helper';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { PaymentConfigurationRepository } from '../payment-configuration/payment-configuration.repository';
import { RefundItemRepository } from '../refund-item/refund-item.repository';
import { CreatePaymentGatewayDto } from './dto/create-payment-gateway.dto';
import { PaymentGatewayRepository } from './payment-gateway.repository';

@Injectable()
export class PaymentGatewayService {
    constructor(
        @InjectRepository(PaymentConfigurationRepository)
        private readonly paymentConfigurationRepository: PaymentConfigurationRepository,
        @InjectRepository(PaymentGatewayRepository)
        private readonly paymentGatewayRepository: PaymentGatewayRepository,
        @InjectRepository(RefundItemRepository)
        private readonly refundItemRepository: RefundItemRepository
    ) {
    }

    async payment(dto: CreatePaymentGatewayDto, request: any): Promise<any | Error> {
        const paymentConf = await this.paymentConfigurationRepository.find();
        if (!paymentConf) {
            throw new NotFoundException("Payment security key not found");
        }
        const dp = new DirectPost(paymentConf[0].securityKey);

        const billingInfo = {
            'first_name': dto.billingInfo.firstName,
            'last_name': dto.billingInfo.lastName,
            'address1': dto.billingInfo.addressOne,
            'city': dto.billingInfo.city,
            'state': dto.billingInfo.state,
            'zip': dto.billingInfo.zip,
        }
        const shippingInfo = {
            'shipping_first_name': dto.shippingInfo.firstName,
            'shipping_last_name': dto.shippingInfo.lastName,
            'shipping_address1': dto.shippingInfo.addressOne,
            'shipping_city': dto.shippingInfo.city,
            'shipping_state': dto.shippingInfo.state,
            'shipping_zip': dto.shippingInfo.zip,
        }

        dp.setBilling(billingInfo);
        dp.setShipping(shippingInfo);
        const data = await dp.doSale(dto.amount, dto.ccNum, dto.ccExp, dto.cvv);

        if (data['statusCode'] === 200 && data['response'] === "1" && data['response_code'] === "100") {
            const payload = {
                customerId: request.user.id,
                customerName: request.user.firstName + ' ' + request.user.lastName,
                invoiceID: dto.invoiceID,
                paymentAmount: dto.amount,
                paymentDate: new Date(),
                paymentTransID: data['transactionid'],
                paymentPayload: JSON.stringify(data),
                transStatus: "Success",
            }
            const res = await this.paymentGatewayRepository.createEntity(payload);
            return ResponseUtils.successResponseHandler(200, "Payment done successfully", "data", res);
        } else return ResponseUtils.errorResponseHandler(400, "Error Occuered!", "data", data);
    }

    async refund(id: string, dto: any): Promise<any | Error> {
        const paymentConf = await this.paymentConfigurationRepository.find();
        if (!paymentConf) {
            throw new NotFoundException(Constants.NOT_FOUND);
        }

        const paymentGateway = await this.paymentGatewayRepository.findOne({
            where: {
                id: id,
                isDeleted: false
            }
        })
        if (!paymentGateway) {
            throw new NotFoundException(Constants.NOT_FOUND);
        }

        const dp = new DirectPost(paymentConf[0].securityKey);
        const data = await dp.doRefund(dto.amount, dto.transactionID);
        if (data['statusCode'] === 200 && data['response'] === "1" && data['response_code'] === "100") {

            let amountDto = Number(dto.amount);
            let previousRefundAmount = Number(paymentGateway.refundAmount);

            const payloadAmount = (amountDto - 0) + (previousRefundAmount - 0);

            const payload = {
                refundTransID: data['transactionid'],
                refundPayload: JSON.stringify(data),
                isRefunded: true,
                refundDate: new Date(),
                refundAmount: payloadAmount,
            }

            const res = await this.paymentGatewayRepository.updateEntity(id, payload);
            if (res) {
                const refundItem = await this.refundItemRepository.createEntity({
                    invoiceID: dto.orderId,
                    refundAmount: dto.amount,
                    isRefunded: true,
                    refundDate: new Date(),
                    refundProductID: id
                })
            }
            return ResponseUtils.successResponseHandler(200, "Refund done successfully", "data", res);
        } else return ResponseUtils.errorResponseHandler(400, "Error Occuered!", "data", data);
    }

    async refundProductPrice(id: string, dto: any): Promise<any | Error> {
        const paymentConf = await this.paymentConfigurationRepository.find();
        if (!paymentConf) {
            throw new NotFoundException(Constants.NOT_FOUND);
        }

        const paymentGateway = await this.paymentGatewayRepository.findOne({
            where: {
                invoiceID: dto.orderId,
                isDeleted: false
            }
        })
        if (!paymentGateway) {
            throw new NotFoundException(Constants.NOT_FOUND);
        }

        const dp = new DirectPost(paymentConf[0].securityKey);
        const data = await dp.doRefund(dto.amount, paymentGateway.paymentTransID);
        if (data['statusCode'] === 200 && data['response'] === "1" && data['response_code'] === "100") {
            const payload = {
                refundTransID: data['transactionid'],
                refundPayload: JSON.stringify(data),
                isRefunded: true,
                refundDate: new Date(),
                refundAmount: parseFloat(dto.amount + Number(paymentGateway.refundAmount)),
            }
            const res = await this.paymentGatewayRepository.updateEntity(paymentGateway.id, payload);
            if (res) {
                const refundItem = await this.refundItemRepository.createEntity({
                    invoiceID: dto.orderId,
                    refundAmount: dto.amount,
                    isRefunded: true,
                    refundDate: new Date(),
                    refundProductID: id
                })
            }
            return ResponseUtils.successResponseHandler(200, "Refund done successfully", "data", res);
        } else return ResponseUtils.errorResponseHandler(400, "Error Occuered!", "data", data);
    }

    async findAll(
        transId: string,
        startDate: Date,
        endDate: Date,
        userName: string,
        orderId: string,
    ): Promise<any[] | Error> {
        let query: any = {};
        if (transId) query.paymentTransID = transId;
        if (startDate) query.startDate = startDate;
        if (endDate) query.endDate = endDate;
        if (userName) query.customerName = userName;
        if (orderId) query.invoiceID = orderId;
        const data = await this.paymentGatewayRepository.findAllEntities(query);

        if (!data) {
            throw new NotFoundException(Constants.NOT_FOUND);
        }

        for (let i = 0; i < Object.keys(data).length; i++) {
            if (data[i].paymentAmount === data[i].refundAmount) {
                data[i]['isGlobalRefunded'] = true;
            } else {
                data[i]['isGlobalRefunded'] = false;
            }
        }

        return ResponseUtils.successResponseHandler(
            200,
            "Payment Log Data Fetched",
            "data",
            data
        );
    }

    async refundReport(
        transId: string,
        startDate: Date,
        endDate: Date,
        userName: string,
        orderId: string,
    ): Promise<any[] | Error> {
        let query: any = {};
        if (transId) query.refundTransID = transId;
        if (startDate) query.startDate = startDate;
        if (endDate) query.endDate = endDate;
        if (userName) query.customerName = userName;
        if (orderId) query.invoiceID = orderId;
        const data = await this.paymentGatewayRepository.refundReportEntities(query);

        if (!data) {
            throw new NotFoundException(Constants.NOT_FOUND);
        }

        return ResponseUtils.successResponseHandler(
            200,
            "Refund Data Fetched",
            "data",
            data
        );
    }
}
