import {
    Body,
    Controller,
    Delete,
    Get,
    Param,
    Patch,
    Post,
    Query,
    UploadedFile,
    UploadedFiles,
    UseInterceptors
} from '@nestjs/common';
import { FileFieldsInterceptor, FileInterceptor } from '@nestjs/platform-express';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { Constants } from 'src/utils/constants';
import { CreateProductDto } from './dto/create-product.dto';
import { UpdateProductDto } from './dto/update-product.dto';
import { ProductService } from './product.service';
import { getMulterConfig } from 'src/core/multer.config';
import { CreateImportProductStockDto } from './dto/create-import-product-stock.dto';
import { CreateConfirmStockInProductDto } from './dto/create-confirm-product-stock.dto';
import { CreateImportProductDto } from './dto/create-import-product.dto';
import { CreateConfirmImportProductDto } from './dto/create-confirm-import-product.dto';

@Controller({
    path: 'product',
    version: Constants.API_VERSION_1
})
export class ProductController {
    constructor(
        private readonly productsService: ProductService
    ) {}

    @Post()
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: 'productImages', maxCount: 10 },
            { name: 'featuredImage', maxCount: 1 }
        ]
    ))
    create(
        @Body() dto: CreateProductDto,
        @UploadedFiles() files: {
            productImages?: UploadedMulterFileI[],
            featuredImage?: UploadedMulterFileI
        }
    ) {
        return this.productsService.create(dto, files);
    }

    @Get()
    async findAll(
        @Query('limit') limit: number,
        @Query('page') page: number
    ) {
        return await this.productsService.findAll(limit, page);
    }

    @Patch('/:id')
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: 'productImages', maxCount: 10 },
            { name: 'featuredImage', maxCount: 1 }
        ]
    ))
    async updateProduct(
        @Param('id') id: string,
        @Body() updateProductDto: UpdateProductDto,
        @UploadedFiles() files: {
            productImages?: UploadedMulterFileI[],
            featuredImage?: UploadedMulterFileI
        }
    ) {
        return await this.productsService.updateOneProduct(id, updateProductDto, files);
    }

    @Delete(':id')
    async remove(@Param('id') id: string) {
        return await this.productsService.remove(id);
    }

    @Get('/:id')
    async findOne(@Param('id') id: string) {
        return await this.productsService.findOne(id);
    }

    @Get('/product-details/:slug')
    async findOneBySlug(@Param('slug') slug: string) {
        return await this.productsService.findOneBySlug(slug);
    }

    @Post('/new-stock-in')
    @UseInterceptors(
        FileInterceptor(
            'excelFile',
            getMulterConfig(
                './uploads/product-stockin'
            )
        )
    )
    importStockInProduct(@Body() importProductStockDto: CreateImportProductStockDto, @UploadedFile() file: Express.Multer.File) {
        return this.productsService.importProductStockService(importProductStockDto, file);
    }

    @Post('/confirm-stock-in-product')
    confirmStockInProduct(@Body() dto: CreateConfirmStockInProductDto) {
        return this.productsService.confirmStockInProductService(dto);
    }

    @Post('/import-product')
    @UseInterceptors(
        FileInterceptor(
            'excelFile',
            getMulterConfig(
                './uploads/product-dump'
            )
        )
    )
    importProduct(@Body() importProductDto: CreateImportProductDto, @UploadedFile() file: Express.Multer.File) {
        return this.productsService.importProductService(importProductDto, file);
    }

    @Post('/confirm-import-product')
    confirmImportProduct(@Body() dto: CreateConfirmImportProductDto) {
        return this.productsService.confirmImportProductService(dto);
    }

    @Get('/search/:name')
    async searchProductByName(
        @Param('name') name: string
    ) {
        return await this.productsService.searchProductByName(name);
    }
}
